@@ -53,6 +53,7 @@ struct EnumInfo {
     module: Option<ModuleName>,
 }
 
+/// A struct or enum variant, with named fields.
 struct FieldContainer {
     name: syn::Ident,
     backtrace_field: Option<Field>,
@@ -63,6 +64,7 @@ struct FieldContainer {
     visibility: Option<UserInput>,
     module: Option<ModuleName>,
     provides: Vec<Provide>,
+    is_transparent: bool,
 }
 
 impl FieldContainer {
@@ -84,6 +86,26 @@ struct Provide {
     expr: syn::Expr,
 }
 
+enum ContextSelectorName {
+    Provided(syn::Ident),
+    Suffixed(SuffixKind),
+}
+
+impl Default for ContextSelectorName {
+    fn default() -> Self {
+        ContextSelectorName::Suffixed(SuffixKind::Default)
+    }
+}
+
+impl ContextSelectorName {
+    fn resolve_name(&self, def: &SuffixKind, base_name: &syn::Ident) -> syn::Ident {
+        match self {
+            ContextSelectorName::Provided(ident) => ident.clone(),
+            ContextSelectorName::Suffixed(suffix_kind) => suffix_kind.resolve_name(def, base_name),
+        }
+    }
+}
+
 enum SuffixKind {
     Default,
     None,
@@ -91,20 +113,34 @@ enum SuffixKind {
 }
 
 impl SuffixKind {
-    fn resolve_with_default<'a>(&'a self, def: &'a Self) -> &'a Self {
-        use SuffixKind::*;
+    const DEFAULT_SUFFIX: &'static str = "Snafu";
 
+    fn resolve_name(&self, def: &Self, base_name: &syn::Ident) -> syn::Ident {
+        let span = base_name.span();
+
+        let base_name = base_name.to_string();
+        let base_name = base_name.trim_end_matches("Error");
+
+        let suffix = self
+            .as_option()
+            .or_else(|| def.as_option())
+            .unwrap_or(&Self::DEFAULT_SUFFIX);
+
+        quote::format_ident!("{}{}", base_name, suffix, span = span)
+    }
+
+    fn as_option(&self) -> Option<&dyn quote::IdentFragment> {
         match self {
-            Default => def,
-            None => self,
-            Some(_) => self,
+            SuffixKind::Default => None,
+            SuffixKind::None => Some(&""),
+            SuffixKind::Some(s) => Some(s),
         }
     }
 }
 
 enum ContextSelectorKind {
     Context {
-        suffix: SuffixKind,
+        selector_name: ContextSelectorName,
         source_field: Option<SourceField>,
         user_fields: Vec<Field>,
     },
@@ -121,10 +157,7 @@ enum ContextSelectorKind {
 
 impl ContextSelectorKind {
     fn is_whatever(&self) -> bool {
-        match self {
-            ContextSelectorKind::Whatever { .. } => true,
-            _ => false,
-        }
+        matches!(self, ContextSelectorKind::Whatever { .. })
     }
 
     fn user_fields(&self) -> &[Field] {
@@ -150,6 +183,20 @@ impl ContextSelectorKind {
             ContextSelectorKind::NoContext { .. } => None,
         }
     }
+
+    fn resolve_name(&self, def: &SuffixKind, base_name: &syn::Ident) -> syn::Ident {
+        let selector_name_default;
+
+        let selector_name = match self {
+            ContextSelectorKind::Context { selector_name, .. } => selector_name,
+            _ => {
+                selector_name_default = ContextSelectorName::default();
+                &selector_name_default
+            }
+        };
+
+        selector_name.resolve_name(def, base_name)
+    }
 }
 
 struct NamedStructInfo {
@@ -299,6 +346,7 @@ enum ErrorLocation {
     OnNamedStruct,
     InNamedStruct,
     OnTupleStruct,
+    OnTupleStructField,
 }
 
 impl fmt::Display for ErrorLocation {
@@ -313,6 +361,7 @@ impl fmt::Display for ErrorLocation {
             OnNamedStruct => "on a named struct".fmt(f),
             InNamedStruct => "within a named struct".fmt(f),
             OnTupleStruct => "on a tuple struct".fmt(f),
+            OnTupleStructField => "on a tuple struct field".fmt(f),
         }
     }
 }
@@ -335,6 +384,20 @@ impl SyntaxErrorsScoped<'_> {
     }
 }
 
+/// Helper structure to handle cases where an attribute is
+/// syntactically valid but semantically invalid.
+#[derive(Debug)]
+struct DoesNothing {
+    /// The name of the attribute that was misused.
+    attribute: &'static str,
+}
+
+impl ErrorForLocation for DoesNothing {
+    fn for_location(&self, _location: ErrorLocation) -> String {
+        format!("`{}` attribute has no effect", self.attribute)
+    }
+}
+
 /// Helper structure to handle cases where an attribute was used on an
 /// element where it's not valid.
 #[derive(Debug)]
@@ -575,9 +638,8 @@ const ATTR_IMPLICIT: OnlyValidOn = OnlyValidOn {
     valid_on: "enum variant or struct fields with a name",
 };
 
-const ATTR_IMPLICIT_FALSE: WrongField = WrongField {
+const ATTR_IMPLICIT_FALSE: DoesNothing = DoesNothing {
     attribute: "implicit(false)",
-    valid_field: "location",
 };
 
 const ATTR_VISIBILITY: OnlyValidOn = OnlyValidOn {
@@ -615,6 +677,11 @@ const ATTR_CONTEXT_FLAG: OnlyValidOn = OnlyValidOn {
     valid_on: "enum variants or structs with named fields",
 };
 
+const ATTR_CONTEXT_NAME: OnlyValidOn = OnlyValidOn {
+    attribute: "context(name)",
+    valid_on: "enum variants or structs with named fields",
+};
+
 const ATTR_WHATEVER: OnlyValidOn = OnlyValidOn {
     attribute: "whatever",
     valid_on: "enum variants or structs with named fields",
@@ -625,6 +692,15 @@ const ATTR_CRATE_ROOT: OnlyValidOn = OnlyValidOn {
     valid_on: "an enum or a struct",
 };
 
+const ATTR_TRANSPARENT: OnlyValidOn = OnlyValidOn {
+    attribute: "transparent",
+    valid_on: "enum variants or structs with named fields",
+};
+
+const ATTR_TRANSPARENT_FALSE: DoesNothing = DoesNothing {
+    attribute: "transparent(false)",
+};
+
 const SOURCE_BOOL_FROM_INCOMPATIBLE: IncompatibleAttributes =
     IncompatibleAttributes(&["source(false)", "source(from)"]);
 
@@ -663,6 +739,7 @@ fn parse_snafu_enum(
             Att::Context(tokens, c) => match c {
                 Context::Suffix(s) => default_suffixes.add(s, tokens),
                 Context::Flag(_) => enum_errors.add(tokens, ATTR_CONTEXT_FLAG),
+                Context::Name(_) => enum_errors.add(tokens, ATTR_CONTEXT_NAME),
             },
             Att::Module(tokens, v) => modules.add(v, tokens),
             Att::Provide(tokens, ProvideKind::Flag(..)) => {
@@ -673,6 +750,7 @@ fn parse_snafu_enum(
             }
             Att::Backtrace(tokens, ..) => enum_errors.add(tokens, ATTR_BACKTRACE),
             Att::Implicit(tokens, ..) => enum_errors.add(tokens, ATTR_IMPLICIT),
+            Att::Transparent(tokens, ..) => enum_errors.add(tokens, ATTR_TRANSPARENT),
             Att::Whatever(tokens) => enum_errors.add(tokens, ATTR_WHATEVER),
             Att::DocComment(..) => { /* Just a regular doc comment. */ }
         }
@@ -758,6 +836,8 @@ fn field_container(
 
     let mut contexts = AtMostOne::new("context", outer_error_location);
     let mut whatevers = AtMostOne::new("whatever", outer_error_location);
+    let mut transparents = AtMostOne::new("transparent", outer_error_location);
+
     let mut doc_comment = DocComment::default();
     let mut reached_end_of_doc_comment = false;
 
@@ -770,6 +850,13 @@ fn field_container(
             Att::Visibility(tokens, v) => visibilities.add(v, tokens),
             Att::Context(tokens, c) => contexts.add(c, tokens),
             Att::Whatever(tokens) => whatevers.add((), tokens),
+            Att::Transparent(tokens, t) => {
+                if t {
+                    transparents.add((), tokens)
+                } else {
+                    outer_errors.add(tokens, ATTR_TRANSPARENT_FALSE)
+                }
+            }
             Att::Source(tokens, ..) => outer_errors.add(tokens, ATTR_SOURCE),
             Att::Backtrace(tokens, ..) => outer_errors.add(tokens, ATTR_BACKTRACE),
             Att::Implicit(tokens, ..) => outer_errors.add(tokens, ATTR_IMPLICIT),
@@ -827,7 +914,6 @@ fn field_container(
         // exclude fields even if they have the "source" or "backtrace" name.
         let mut source_opt_out = false;
         let mut backtrace_opt_out = false;
-        let mut implicit_opt_out = false;
         let mut provide_opt_out = false;
 
         let mut field_errors = errors.scoped(ErrorLocation::OnField);
@@ -878,8 +964,6 @@ fn field_container(
                 Att::Implicit(tokens, v) => {
                     if v {
                         implicit_attrs.add((), tokens);
-                    } else if is_implicit_location(name) {
-                        implicit_opt_out = true;
                     } else {
                         field_errors.add(tokens, ATTR_IMPLICIT_FALSE);
                     }
@@ -900,6 +984,7 @@ fn field_container(
                 Att::Visibility(tokens, ..) => field_errors.add(tokens, ATTR_VISIBILITY),
                 Att::Display(tokens, ..) => field_errors.add(tokens, ATTR_DISPLAY),
                 Att::Context(tokens, ..) => field_errors.add(tokens, ATTR_CONTEXT),
+                Att::Transparent(tokens, ..) => field_errors.add(tokens, ATTR_TRANSPARENT),
                 Att::Whatever(tokens) => field_errors.add(tokens, ATTR_WHATEVER),
                 Att::CrateRoot(tokens, ..) => field_errors.add(tokens, ATTR_CRATE_ROOT),
                 Att::DocComment(..) => { /* Just a regular doc comment. */ }
@@ -921,7 +1006,7 @@ fn field_container(
         let field = Field {
             name: name.clone(),
             ty: syn_field.ty.clone(),
-            provide: provide_attr.is_some() || (is_implicit_provide(&name) && !provide_opt_out),
+            provide: provide_attr.is_some() || (is_implicit_provide(name) && !provide_opt_out),
             original,
         };
 
@@ -941,8 +1026,7 @@ fn field_container(
             }
         });
 
-        let implicit_attr =
-            implicit_attr.is_some() || (is_implicit_location(&field.name) && !implicit_opt_out);
+        let implicit_attr = implicit_attr.is_some();
 
         if let Some((maybe_transformation, location)) = source_attr {
             let Field {
@@ -1001,19 +1085,51 @@ fn field_container(
     let (module, errs) = modules.finish();
     errors.extend(errs);
 
-    let (display_format, errs) = display_formats.finish();
+    let (display_format, errs) = display_formats.finish_with_location();
     errors.extend(errs);
 
     let (visibility, errs) = visibilities.finish();
     errors.extend(errs);
 
     let (is_context, errs) = contexts.finish_with_location();
-    let is_context = is_context.map(|(c, tt)| (c.into_enabled(), tt));
+    let mut is_context = is_context.map(|(c, tt)| (c.into_enabled(), tt));
+    let mut is_context_false_checks_source = "Context selectors without context";
     errors.extend(errs);
 
     let (is_whatever, errs) = whatevers.finish_with_location();
     errors.extend(errs);
 
+    let (is_transparent, errs) = transparents.finish_with_location();
+    errors.extend(errs);
+
+    if let (Some((_, d_tt)), Some((_, t_tt))) = (&display_format, &is_transparent) {
+        let txt = "`transparent` errors cannot have a display format because they delegate `Display` to their source";
+        errors.extend([
+            syn::Error::new_spanned(d_tt, txt),
+            syn::Error::new_spanned(t_tt, txt),
+        ]);
+    }
+
+    match (&is_context, &is_transparent) {
+        (Some(((true, _), c_tt)), Some((_, t_tt))) => {
+            let txt = "`transparent` errors cannot have context";
+            errors.extend([
+                syn::Error::new_spanned(c_tt, txt),
+                syn::Error::new_spanned(t_tt, txt),
+            ]);
+        }
+        (None, Some((_, t_tt))) => {
+            // `transparent` implies `context(false)`. Treat `transparent` as if setting
+            // `context(false)` as well.
+            is_context = Some((
+                (false, ContextSelectorName::Suffixed(SuffixKind::None)),
+                t_tt.clone(),
+            ));
+            is_context_false_checks_source = "`transparent` errors";
+        }
+        (Some(((false, _), _)), Some(_)) | (_, None) => {}
+    }
+
     let source_field = source.map(|(val, _tts)| val);
 
     let selector_kind = match (is_context, is_whatever) {
@@ -1026,13 +1142,13 @@ fn field_container(
         }
 
         (Some(((true, suffix), _)), None) => ContextSelectorKind::Context {
-            suffix,
+            selector_name: suffix,
             source_field,
             user_fields,
         },
 
         (None, None) => ContextSelectorKind::Context {
-            suffix: SuffixKind::Default,
+            selector_name: ContextSelectorName::Suffixed(SuffixKind::Default),
             source_field,
             user_fields,
         },
@@ -1073,14 +1189,20 @@ fn field_container(
             errors.extend(user_fields.into_iter().map(|Field { original, .. }| {
                 syn::Error::new_spanned(
                     original,
-                    "Context selectors without context must not have context fields",
+                    format_args!(
+                        "{} must not have context fields",
+                        is_context_false_checks_source
+                    ),
                 )
             }));
 
             let source_field = source_field.ok_or_else(|| {
                 vec![syn::Error::new(
                     variant_span,
-                    "Context selectors without context must have a source field",
+                    format_args!(
+                        "{} must have a source field",
+                        is_context_false_checks_source
+                    ),
                 )]
             })?;
 
@@ -1093,18 +1215,18 @@ fn field_container(
         backtrace_field: backtrace.map(|(val, _tts)| val),
         implicit_fields,
         selector_kind,
-        display_format,
+        display_format: display_format.map(|(d, _)| d),
         doc_comment: doc_comment.finish(),
         visibility,
         module,
         provides,
+        is_transparent: is_transparent.is_some(),
     })
 }
 
 const IMPLICIT_SOURCE_FIELD_NAME: &str = "source";
 const IMPLICIT_BACKTRACE_FIELD_NAME: &str = "backtrace";
 const IMPLICIT_MESSAGE_FIELD_NAME: &str = "message";
-const IMPLICIT_LOCATION_FIELD_NAME: &str = "location";
 
 fn is_implicit_source(name: &proc_macro2::Ident) -> bool {
     name == IMPLICIT_SOURCE_FIELD_NAME
@@ -1118,10 +1240,6 @@ fn is_implicit_message(name: &proc_macro2::Ident) -> bool {
     name == IMPLICIT_MESSAGE_FIELD_NAME
 }
 
-fn is_implicit_location(name: &proc_macro2::Ident) -> bool {
-    name == IMPLICIT_LOCATION_FIELD_NAME
-}
-
 fn is_implicit_provide(name: &proc_macro2::Ident) -> bool {
     is_implicit_source(name) || is_implicit_backtrace(name)
 }
@@ -1233,7 +1351,8 @@ fn parse_snafu_tuple_struct(
             Att::Backtrace(tokens, ..) => struct_errors.add(tokens, ATTR_BACKTRACE),
             Att::Implicit(tokens, ..) => struct_errors.add(tokens, ATTR_IMPLICIT),
             Att::Context(tokens, ..) => struct_errors.add(tokens, ATTR_CONTEXT),
-            Att::Whatever(tokens) => struct_errors.add(tokens, ATTR_CONTEXT),
+            Att::Whatever(tokens) => struct_errors.add(tokens, ATTR_WHATEVER),
+            Att::Transparent(tokens, ..) => struct_errors.add(tokens, ATTR_TRANSPARENT),
             Att::CrateRoot(tokens, root) => crate_roots.add(root, tokens),
             Att::DocComment(..) => { /* Just a regular doc comment. */ }
         }
@@ -1254,7 +1373,30 @@ fn parse_snafu_tuple_struct(
         return Err(vec![one_field_error(span)]);
     }
 
-    let ty = inner.into_value().ty;
+    let inner = inner.into_value();
+
+    let mut field_errors = errors.scoped(ErrorLocation::OnTupleStructField);
+
+    for attr in attributes_from_syn(inner.attrs)? {
+        use SnafuAttribute as Att;
+
+        match attr {
+            Att::Backtrace(tokens, ..) => field_errors.add(tokens, ATTR_BACKTRACE),
+            Att::Context(tokens, ..) => field_errors.add(tokens, ATTR_CONTEXT),
+            Att::CrateRoot(tokens, ..) => field_errors.add(tokens, ATTR_CRATE_ROOT),
+            Att::Display(tokens, ..) => field_errors.add(tokens, ATTR_DISPLAY),
+            Att::Implicit(tokens, ..) => field_errors.add(tokens, ATTR_IMPLICIT),
+            Att::Module(tokens, ..) => field_errors.add(tokens, ATTR_MODULE),
+            Att::Provide(tokens, ..) => field_errors.add(tokens, ATTR_PROVIDE_FLAG),
+            Att::Source(tokens, ..) => field_errors.add(tokens.clone(), ATTR_SOURCE),
+            Att::Transparent(tokens, ..) => field_errors.add(tokens, ATTR_TRANSPARENT),
+            Att::Visibility(tokens, ..) => field_errors.add(tokens, ATTR_VISIBILITY),
+            Att::Whatever(tokens) => field_errors.add(tokens, ATTR_WHATEVER),
+            Att::DocComment(..) => { /* Just a regular doc comment. */ }
+        }
+    }
+
+    let ty = inner.ty;
     let (maybe_transformation, errs) = transformations.finish();
     let transformation = maybe_transformation
         .map(|(source_ty, expr)| Transformation::Transform {
@@ -1282,14 +1424,16 @@ fn parse_snafu_tuple_struct(
 
 enum Context {
     Flag(bool),
+    Name(syn::Ident),
     Suffix(SuffixKind),
 }
 
 impl Context {
-    fn into_enabled(self) -> (bool, SuffixKind) {
+    fn into_enabled(self) -> (bool, ContextSelectorName) {
         match self {
-            Context::Flag(b) => (b, SuffixKind::None),
-            Context::Suffix(suffix) => (true, suffix),
+            Context::Flag(b) => (b, ContextSelectorName::Suffixed(SuffixKind::None)),
+            Context::Name(name) => (true, ContextSelectorName::Provided(name)),
+            Context::Suffix(suffix) => (true, ContextSelectorName::Suffixed(suffix)),
         }
     }
 }
@@ -1314,7 +1458,7 @@ struct DocComment {
 impl DocComment {
     fn push_str(&mut self, s: &str) {
         if !self.content.is_empty() {
-            self.content.push_str(" ");
+            self.content.push(' ');
         }
         self.content.push_str(s);
     }
@@ -1350,6 +1494,7 @@ enum SnafuAttribute {
     Module(proc_macro2::TokenStream, ModuleName),
     Provide(proc_macro2::TokenStream, ProvideKind),
     Source(proc_macro2::TokenStream, Vec<Source>),
+    Transparent(proc_macro2::TokenStream, bool),
     Visibility(proc_macro2::TokenStream, UserInput),
     Whatever(proc_macro2::TokenStream),
 }
@@ -1410,9 +1555,10 @@ trait GenericAwareNames {
 
     fn provided_generic_types_without_defaults(&self) -> Vec<proc_macro2::TokenStream> {
         use syn::TypeParam;
+
         self.generics()
             .type_params()
-            .map(|t: &TypeParam| {
+            .map(|t| {
                 let TypeParam {
                     attrs,
                     ident,
@@ -1433,32 +1579,60 @@ trait GenericAwareNames {
     fn provided_generics_without_defaults(&self) -> Vec<proc_macro2::TokenStream> {
         self.provided_generic_lifetimes()
             .into_iter()
-            .chain(self.provided_generic_types_without_defaults().into_iter())
+            .chain(self.provided_generic_types_without_defaults())
+            .chain(self.provided_generic_consts_without_defaults())
             .collect()
     }
 
     fn provided_generic_lifetimes(&self) -> Vec<proc_macro2::TokenStream> {
-        use syn::{GenericParam, LifetimeDef};
+        use syn::LifetimeParam;
 
         self.generics()
-            .params
-            .iter()
-            .flat_map(|p| match p {
-                GenericParam::Lifetime(LifetimeDef { lifetime, .. }) => Some(quote! { #lifetime }),
-                _ => None,
+            .lifetimes()
+            .map(|l| {
+                let LifetimeParam {
+                    attrs, lifetime, ..
+                } = l;
+                quote! {
+                    #(#attrs)*
+                    #lifetime
+                }
+            })
+            .collect()
+    }
+
+    fn provided_generic_consts_without_defaults(&self) -> Vec<proc_macro2::TokenStream> {
+        self.generics()
+            .const_params()
+            .map(|c| {
+                let syn::ConstParam {
+                    attrs,
+                    const_token,
+                    ident,
+                    colon_token,
+                    ty,
+                    ..
+                } = c;
+                quote! {
+                    #(#attrs)*
+                    #const_token
+                    #ident
+                    #colon_token
+                    #ty
+                }
             })
             .collect()
     }
 
     fn provided_generic_names(&self) -> Vec<proc_macro2::TokenStream> {
-        use syn::{ConstParam, GenericParam, LifetimeDef, TypeParam};
+        use syn::{ConstParam, GenericParam, LifetimeParam, TypeParam};
 
         self.generics()
             .params
             .iter()
             .map(|p| match p {
                 GenericParam::Type(TypeParam { ident, .. }) => quote! { #ident },
-                GenericParam::Lifetime(LifetimeDef { lifetime, .. }) => quote! { #lifetime },
+                GenericParam::Lifetime(LifetimeParam { lifetime, .. }) => quote! { #lifetime },
                 GenericParam::Const(ConstParam { ident, .. }) => quote! { #ident },
             })
             .collect()
@@ -1575,9 +1749,9 @@ impl<'a> quote::ToTokens for ContextSelector<'a> {
             original_generics_without_defaults: &self.0.provided_generics_without_defaults(),
             parameterized_error_name: &self.0.parameterized_name(),
             selector_doc_string: &selector_doc_string,
-            selector_kind: &selector_kind,
-            selector_name: variant_name,
-            user_fields: &selector_kind.user_fields(),
+            selector_kind,
+            selector_base_name: variant_name,
+            user_fields: selector_kind.user_fields(),
             visibility: selector_visibility,
             where_clauses: &self.0.provided_where_clauses(),
             default_suffix,
@@ -1762,7 +1936,7 @@ impl NamedStructInfo {
         };
 
         let error_source_match_arm = ErrorSourceMatchArm {
-            field_container: &field_container,
+            field_container,
             pattern_ident,
         };
         let error_source_match_arm = quote! { #error_source_match_arm };
@@ -1810,7 +1984,7 @@ impl NamedStructInfo {
             display_format: display_format.as_ref(),
             doc_comment: doc_comment.as_ref(),
             pattern_ident: &quote! { Self },
-            selector_kind: &selector_kind,
+            selector_kind,
         };
         let arm = quote! { #arm };
 
@@ -1837,15 +2011,15 @@ impl NamedStructInfo {
 
         let context_selector = ContextSelector {
             backtrace_field: backtrace_field.as_ref(),
-            implicit_fields: implicit_fields,
+            implicit_fields,
             crate_root: &crate_root,
             error_constructor_name: &name,
             original_generics_without_defaults: &original_generics,
             parameterized_error_name: &parameterized_struct_name,
             selector_doc_string: &selector_doc_string,
-            selector_kind: &selector_kind,
-            selector_name: &field_container.name,
-            user_fields: &user_fields,
+            selector_kind,
+            selector_base_name: &field_container.name,
+            user_fields,
             visibility: selector_visibility,
             where_clauses: &where_clauses,
             default_suffix: &SuffixKind::Default,
@@ -1931,28 +2105,18 @@ impl TupleStructInfo {
             }
         };
 
-        let std_backtrace_fn = if cfg!(feature = "unstable-backtraces-impl-std") {
-            quote! {
-                fn backtrace(&self) -> ::core::option::Option<&std::backtrace::Backtrace> {
-                    #crate_root::ErrorCompat::backtrace(self)
-                }
-            }
-        } else {
-            quote! {}
-        };
-
         let provide_fn = if cfg!(feature = "unstable-provider-api") {
             use shared::error::PROVIDE_ARG;
 
             let provides = shared::error::enhance_provider_list(&provides);
             let cached_expressions = shared::error::quote_cached_expressions(&provides);
-            let user_chained = shared::error::quote_chained(&provides);
+            let user_chained = shared::error::quote_chained(&crate_root, &provides);
 
             let (hi_explicit_calls, lo_explicit_calls) =
                 shared::error::build_explicit_provide_calls(&provides);
 
             Some(quote! {
-                fn provide<'a>(&'a self, #PROVIDE_ARG: &mut core::any::Demand<'a>) {
+                fn provide<'a>(&'a self, #PROVIDE_ARG: &mut #crate_root::error::Request<'a>) {
                     match self {
                         Self(v) => {
                             #(#cached_expressions;)*
@@ -1977,7 +2141,6 @@ impl TupleStructInfo {
                 #description_fn
                 #cause_fn
                 #source_fn
-                #std_backtrace_fn
                 #provide_fn
             }
         };
@@ -2034,20 +2197,6 @@ impl GenericAwareNames for TupleStructInfo {
     }
 }
 
-trait Transpose<T, E> {
-    fn my_transpose(self) -> Result<Option<T>, E>;
-}
-
-impl<T, E> Transpose<T, E> for Option<Result<T, E>> {
-    fn my_transpose(self) -> Result<Option<T>, E> {
-        match self {
-            Some(Ok(v)) => Ok(Some(v)),
-            Some(Err(e)) => Err(e),
-            None => Ok(None),
-        }
-    }
-}
-
 mod sponge {
     use std::iter::FromIterator;
 
